package com.jz.sms.chuanglan.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.jz.sms.api.BizController;
import com.jz.sms.chuanglan.dto.*;
import com.jz.sms.chuanglan.service.ChuangLanSmsService;
import com.jz.sms.chuanglan.utils.ChuangLanSmsUtil;
import com.jz.sms.chuanglan.config.ChuangLanSMSConfig;
import com.jz.sms.config.ResponseCode;
import com.jz.sms.repository.domain.SmsTemplateInfo;
import com.jz.sms.service.SmsTemplateService;
import com.jz.sms.util.HttpUtil;
import com.jz.sms.util.Md5Util;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


@SuppressWarnings("DuplicatedCode")
@Service
public class ChuangLanSmsServiceImpl implements ChuangLanSmsService {

    private static final Logger log = LoggerFactory.getLogger(BizController.class);
    private static final String SUCCESS = "success";
    private static final String ERROR = "error";
    private static final String CODE = "code";
    private static final String DATA = "data";

    @Autowired
    private SmsTemplateService templateService;

    @Override
    public CLBizResponse createTemplate(String content, String type) throws Exception {
        int i = 1 / 0;
        log.info("进入创蓝创建短信接口模板的逻辑：{}", content);
        Map<String, Object> requestMap = new HashMap<>();
        String appId;
        String username = ChuangLanSMSConfig.interfaceUser;
        String password = ChuangLanSMSConfig.interfacePassword;
        if ("notification".equals(type)) {
            appId = "49";
        } else if ("marketing".equals(type)) {
            appId = "52";
        } else {
            // TODO 自定义异常,发送警告
            log.error("错误，传入的模板类型未知，类型为：{}", type);
            return new CLBizResponse(ResponseCode.SYSTEM_ERROR, "未知的模板类型:" + type, 0);
        }

        content = content.replaceAll("回TD退订", "");
        String timestamp = String.valueOf(System.currentTimeMillis() / 1000);
        String encrypt = Md5Util.encrypt(new StringBuffer(username).append(password).append(timestamp).toString());
        requestMap.put("username", username);
        requestMap.put("timestamp", timestamp);
        requestMap.put("signature", encrypt);
        requestMap.put("appid", appId);
        requestMap.put("unsubscribe", "回TD退订");
        requestMap.put("content", content);
        log.info("请求创蓝创建短信模板的请求参数为：{}", requestMap);
        String retStr = HttpUtil.doPost(ChuangLanSMSConfig.addTemplate, requestMap);
        log.info("请求创蓝创建短信模板的返回数据为：{}", retStr);
        JSONObject retJson = JSONObject.parseObject(retStr);
        String status = retJson.getString("status");
        if (SUCCESS.equals(status)) {
            JSONObject data = retJson.getJSONObject("data");
            return new CLBizResponse(ResponseCode.SUCCESS, data);
        } else {
            String msg = retJson.getString("msg");
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), msg);
        }
    }

    @Override
    public CLBizResponse sendByFixed(String batchId, String content, String sendParams, String type) throws Exception {
        String account;
        String password;
        if ("notification".equals(type)) {
            account = ChuangLanSMSConfig.notificationAccount;
            password = ChuangLanSMSConfig.notificationPassword;
            log.info("发送短信的模式为：{通知类}");
        } else if ("marketing".equals(type)) {
            account = ChuangLanSMSConfig.marketingAccount;
            password = ChuangLanSMSConfig.marketingPassword;
            log.info("发送短信的模式为：{营销类}");
        } else {
            log.info("发送短信的模式为：{未知类型}");
            throw new Exception("123");
        }

        content = "【九章数据】" + content;
        CLSendFixedRequest request = new CLSendFixedRequest(account, password, content, sendParams, "true", batchId);

        String requestJson = JSONObject.toJSONString(request);
        log.info("【{}】发送固定内容短信,请求上游数据为：{}", batchId, requestJson);
        String responseStr = ChuangLanSmsUtil.sendSmsByPost(ChuangLanSMSConfig.sendFixed, requestJson);
        log.info("【{}】发送固定内容短信,上游返回数据为：{}", batchId, requestJson);

        if (StringUtils.isBlank(responseStr)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        CLSendSMSResponse response = JSON.parseObject(responseStr, CLSendSMSResponse.class);

        String code = response.getCode();
        String errorMsg = response.getErrorMsg();
        String msgId = response.getMsgId();
        if (!"0".equals(code)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), errorMsg);
        } else {
            JSONObject data = new JSONObject();
            data.put("msgId", msgId);
            return new CLBizResponse(ResponseCode.SUCCESS, data);
        }
    }

    @Override
    public CLBizResponse sendByVariable(String batchId, String content, String params, String type) throws Exception {
        String account;
        String password;
        if ("notification".equals(type)) {
            account = ChuangLanSMSConfig.notificationAccount;
            password = ChuangLanSMSConfig.notificationPassword;
            log.info("发送短信的模式为：{通知类}");
        } else if ("marketing".equals(type)) {
            account = ChuangLanSMSConfig.marketingAccount;
            password = ChuangLanSMSConfig.marketingPassword;
            log.info("发送短信的模式为：{营销类}");
        } else {
            log.info("发送短信的模式为：{未知类型}");
            throw new Exception("123");
        }
        content = "【九章数据】" + content;

        CLSendVariableRequest request = new CLSendVariableRequest(account, password, content, params, "true", batchId);

        String requestJson = JSONObject.toJSONString(request);
        log.info("【{}】发送固定内容短信,请求上游数据为：{}，\r\n请求地址为：{}", batchId, requestJson, ChuangLanSMSConfig.sendVariable);
        String responseStr = ChuangLanSmsUtil.sendSmsByPost(ChuangLanSMSConfig.sendVariable, requestJson);
        log.info("【{}】发送固定内容短信,上游返回数据为：{}", batchId, responseStr);

        if (StringUtils.isBlank(responseStr)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        CLSendSMSResponse response = JSON.parseObject(responseStr, CLSendSMSResponse.class);
        String code = response.getCode();
        String errorMsg = response.getErrorMsg();
        String successNum = response.getSuccessNum();
        String failNum = response.getFailNum();
        String msgId = response.getMsgId();
        if (!"0".equals(code)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), errorMsg);
        } else {
            JSONObject data = new JSONObject();
            data.put("successNum", successNum);
            data.put("failNum", failNum);
            data.put("msgId", msgId);
            return new CLBizResponse(ResponseCode.SUCCESS, data);
        }
    }


    @Override
    public CLBizResponse asyncNotify(CLNotifyRequest notifyInfo) {
        String action = notifyInfo.getAction();
        if (!"messageModel".equals(action)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_UNKNOWN_MODEL);
        }
        String id = notifyInfo.getId();
        String status = notifyInfo.getAuditStatus();
        if (!"1".equals(status)) {
            String msg = notifyInfo.getAuditReason();
            boolean update = templateService.updateForSet("status = abnormal,up_reject_msg = " + msg, new EntityWrapper<SmsTemplateInfo>().eq("up_template_id", id));
            // TODO 发送钉钉告警
            log.error("短信模板审核不通过，原因：{}", msg);
        }
        return new CLBizResponse(ResponseCode.SUCCESS);
    }


    @Override
    public CLBizResponse report(ReportDto report) {
        String uid = report.getUid();

        return null;

    }


    public static void main(String[] args) {
        String s = "\"亲爱的${name@0!\"\"}，你当前是${stage@10!\"\"}，请点击链接升级 ${surl!'abc.ddd.cvdfa?fadfadfihjfiahkck?daa=${acd}'} 回TD退订\"";
        String p = "((?:\\$\\{surl.*'\\}))|\\$\\{[^}]+\\}";
        Pattern pt = Pattern.compile(p);
        Matcher matcher = pt.matcher(s);
        while (matcher.find()) {
            String group = matcher.group();
            System.err.println(group);
        }

        System.err.println(11);
    }
}
